from manim_studio.server.server import Server
from PyQt6.QtWidgets import QWidget, QInputDialog, QLabel, QVBoxLayout, QPushButton, QLineEdit
from pyngrok import ngrok


class ManimStudioServer(QWidget):
    def __init__(self, communicate, parent=None):
        super().__init__(parent)
        self.setWindowTitle("Manim Studio Server")
        self.communicate = communicate
        password_dialog = QInputDialog()
        password_dialog.setLabelText("Create a password for the server: ")
        password_dialog.setWindowTitle("Manim Studio Server")
        password_dialog.setModal(True)
        password_dialog.exec()
        self.password = password_dialog.textValue()
        self.server = Server("", 5555, self.password, self.communicate)
        self.server.start()
        self.label = QLabel()
        self.label.setText(
            f"Server started at {self.server.host}:{self.server.port}")
        self.ask_label = QLabel()
        self.ask_label.setText(
            "If you want to work collaboratively, you need to have installed ngrok.\nIf you click the button below, ngrok will be started and you will be able to share the link with your mates.")
        self.ask_label.setWordWrap(True)
        self.ask_label.setMargin(10)
        self.ask_label.setIndent(10)
        self.ask_label.setStyleSheet("background-color: #f0f0f0;")
        self.button = QPushButton("Start ngrok")
        self.button.clicked.connect(self.start_ngrok)
        self.layout = QVBoxLayout()
        self.layout.addWidget(self.label)
        self.layout.addWidget(self.ask_label)
        self.layout.addWidget(self.button)
        self.setLayout(self.layout)

    def closeEvent(self, event):
        self.server.close()
        super().closeEvent(event)

    def start_ngrok(self):
        tunnel = ngrok.connect(5555, "tcp")
        self.button.setText("ngrok started")
        self.button.setEnabled(False)
        self.link_label = QLabel()
        self.link_label.setText(
            "ngrok started. You can share the host and port below with your mates.")
        self.link_label.setWordWrap(True)
        self.link_label.setMargin(10)
        self.link_label.setIndent(10)
        self.link_label.setStyleSheet("background-color: #f0f0f0;")
        self.layout.addWidget(self.link_label)
        self.link = QLineEdit()
        host, port = tunnel.public_url[6:].split(":")
        self.link.setText(f"HOST={host}; PORT={port}")
        self.link.setReadOnly(True)
        self.layout.addWidget(self.link)



import socket
from PyQt6.QtCore import QThread

from manim_studio.communicate import Communicate


class Server(QThread):
    def __init__(self, host: str, port: int, password: str, communicate: Communicate):
        super().__init__()
        self.host = host
        self.port = port
        self.password = password
        self.communicate = communicate
        self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.s.bind((self.host, self.port))
        self.s.listen()
        self.clients: list[socket.socket] = []
        self.threads: list[QThread] = []
        self.running = True

    def run(self):
        while self.running:
            client, address = self.s.accept()
            self.clients.append(client)
            self.threads.append(ClientThread(
                client, address, self.password, self.communicate))
            self.threads[-1].start()

    def close(self):
        self.running = False
        for client in self.clients:
            client.send(b"exit")
            client.close()
        for thread in self.threads:
            thread.quit()
            thread.wait()
        self.s.close()


class ClientThread(QThread):
    def __init__(self, client: socket.socket, address: tuple[str, int], password: str, communicate: Communicate):
        super().__init__()
        self.client = client
        self.address = address
        self.password = password
        self.communicate = communicate

    def run(self):
        msg = self.client.recv(1024)
        try:
            msg.decode("utf-8")
        except UnicodeDecodeError:
            self.client.send(b"Wrong password (UnicodeDecodeError)")
            self.client.close()
            return
        if msg.decode("utf-8") == self.password:
            self.client.send(b"Correct password")
            while True:
                msg = self.client.recv(1024)
                if msg.decode("utf-8") == "exit":
                    break
                self.communicate.update_scene.emit(msg.decode("utf-8"))
                self.client.send(b"Code executed")
            self.client.send(b"exit")
            self.client.close()
        else:
            self.client.send(b"Wrong password")
            self.client.close()



import socket
from PyQt6.QtWidgets import QWidget, QLabel, QVBoxLayout, QPushButton, QTextEdit, QMessageBox


class ManimStudioClient(QWidget):
    def __init__(self, host: str, port: int, password: str):
        super().__init__()
        self.setWindowTitle("Manim Studio Client")
        self.host = host
        self.port = port
        self.password = password
        self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.s.connect((self.host, self.port))
        self.s.send(self.password.encode("utf-8"))
        msg = self.s.recv(1024)
        if msg.decode("utf-8") == "Correct password":
            self.label = QLabel()
            self.label.setText(
                f"Connected to {self.host}:{self.port}")
            self.layout = QVBoxLayout()
            self.layout.addWidget(self.label)
            self.code_edit = QTextEdit()
            self.code_edit.setPlaceholderText("Enter the code")
            self.layout.addWidget(self.code_edit)
            self.send_button = QPushButton("Send")
            self.send_button.clicked.connect(self.send_code)
            self.layout.addWidget(self.send_button)
            self.setLayout(self.layout)
        else:
            QMessageBox.information(
                self, "Debug", msg.decode("utf-8"))
            self.close()

    def closeEvent(self, event):
        self.s.send(b"exit")
        self.s.close()
        super().closeEvent(event)

    def send_code(self):
        code = self.code_edit.toPlainText()
        self.s.send(code.encode("utf-8"))
        msg = self.s.recv(1024)
        if msg.decode("utf-8") == "Code executed":
            QMessageBox.information(
                self, "Code executed", "The code has been executed.")
        else:
            QMessageBox.critical(
                self, "Error", "The code could not be executed. Please try again.")
